# lingo - payment and billing system
# Copyright (C) 2024  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.utils.translation import gettext_lazy as _


class PricingError(Exception):
    def __init__(self, details=None):
        self.details = details or {}
        super().__init__()

    def get_error_message(self):
        return str(self)

    def get_error_display(self):
        formats = {
            'decimal': _('decimal'),
        }
        reason = None
        if self.details.get('reason'):
            reasons = {
                'not-found': _('not found'),
                'wrong-kind': _('wrong kind (group: %(check_type_group)s, check type: %(check_type)s)'),
                'not-configured': _(
                    'pricing not configured (group: %(check_type_group)s, check type: %(check_type)s)'
                ),
            }
            reason = reasons.get(self.details['reason']) % {
                'check_type': self.details.get('check_type'),
                'check_type_group': self.details.get('check_type_group'),
            }

        return self.get_error_message() % {
            'category': self.details.get('category'),
            'criterias': ', '.join(
                '%s (%s)' % (v, _('category: %s') % k) for k, v in self.details.get('criterias', {}).items()
            ),
            'pricing': self.details.get('pricing'),
            'wanted': formats.get(self.details.get('wanted')),
            'status': self.details.get('status'),
            'reason': reason,
            'reduction_rate': self.details.get('reduction_rate'),
            'effort_rate_target': self.details.get('effort_rate_target'),
        }


class PricingNotFound(PricingError):
    def get_error_message(self):
        return _('Agenda pricing not found')


class CriteriaConditionNotFound(PricingError):
    def get_error_message(self):
        return _('No matching criteria for category: %(category)s')


class MultipleDefaultCriteriaCondition(PricingError):
    def get_error_message(self):
        return _('Multiple default criteria found for category: %(category)s')


class PricingDataError(PricingError):
    def get_error_message(self):
        return _('Impossible to determine a pricing for criterias: %(criterias)s')


class MinPricingDataError(PricingError):
    def get_error_message(self):
        return _('Impossible to determine a minimal pricing for criterias: %(criterias)s')


class PricingDataFormatError(PricingError):
    def get_error_message(self):
        return _('Pricing is not a %(wanted)s: %(pricing)s')


class MinPricingDataFormatError(PricingError):
    def get_error_message(self):
        return _('Minimal pricing is not a %(wanted)s: %(pricing)s')


class PricingReductionRateError(PricingError):
    def get_error_message(self):
        return _('Impossible to determine a reduction rate')


class PricingReductionRateFormatError(PricingError):
    def get_error_message(self):
        return _('Reduction rate is not a %(wanted)s: %(reduction_rate)s')


class PricingReductionRateValueError(PricingError):
    def get_error_message(self):
        return _('Reduction rate bad value: %(reduction_rate)s')


class PricingEffortRateTargetError(PricingError):
    def get_error_message(self):
        return _('Impossible to determine an effort rate target')


class PricingEffortRateTargetFormatError(PricingError):
    def get_error_message(self):
        return _('Effort rate target is not a %(wanted)s: %(effort_rate_target)s')


class PricingEffortRateTargetValueError(PricingError):
    def get_error_message(self):
        return _('Effort rate target bad value: %(effort_rate_target)s')


class PricingAccountingCodeError(PricingError):
    def get_error_message(self):
        return _('Impossible to determine an accounting code')


class PricingUnknownCheckStatusError(PricingError):
    def get_error_message(self):
        return _('Unknown check status: %(status)s')


class PricingEventNotCheckedError(PricingError):
    def get_error_message(self):
        return _('Event is not checked')


class PricingBookingNotCheckedError(PricingError):
    def get_error_message(self):
        return _('Booking is not checked')


class PricingMultipleBookingError(PricingError):
    def get_error_message(self):
        return _('Multiple booking found')


class PricingBookingCheckTypeError(PricingError):
    def get_error_message(self):
        return _('Check type error: %(reason)s')
